﻿using System;
using System.IO;
using System.Reflection;
using System.Windows;
using Microsoft.Win32;
using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;

namespace WordTestApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
        {
            var wordStream = GetFromEmbeddedResources("VerkoopOfferte.docx");
            var tempImageStream = GetFromEmbeddedResources("IMG_20190517_142735.jpg");

            var imageStream = new MemoryStream();
            tempImageStream.CopyTo(imageStream);
            imageStream.Position = 0;

            var data = new dynamic[]
            {
                imageStream,
                "TEST",
                "TEST",
                "TEST",
                "TEST",
                "TEST",
                "TEST",
                "TEST"
            };

            using (var loadedDocument = new WordDocument(wordStream, FormatType.Automatic))
            {
                IWTable table = null;

                // find the first table in the document
                foreach (WSection section in loadedDocument.Sections)
                {
                    if (section.Tables.Count > 0)
                    {
                        table = section.Tables[0];
                        break;
                    }
                }

                var rowIndex = 0;
                var rowStartIndex = 1;

                if (table != null)
                {
                    var row = table.Rows.Count - 1 < rowIndex + rowStartIndex
                        ? table.AddRow()
                        : table.Rows[rowIndex + rowStartIndex];

                    // TODO: why is this not being applied?
                    row.RowFormat.Borders.Bottom.BorderType = BorderStyle.Thick;

                    for (var cellIndex = 0; cellIndex < data.Length; cellIndex++)
                    {
                        var cellContent = data[cellIndex];
                        var cell = row.Cells[cellIndex];

                        var paragraph = cell.Paragraphs.Count > 0
                            ? cell.Paragraphs[0]
                            : cell.AddParagraph();

                        var cellText = string.Empty;
                        switch (cellContent)
                        {
                            case string text:
                                cellText = text;
                                break;
                            case MemoryStream stream:
                                stream.Position = 0;
                                var picture = paragraph.AppendPicture(stream.ToArray());

                                picture.TextWrappingStyle = TextWrappingStyle.Inline;
                                picture.VerticalAlignment = ShapeVerticalAlignment.Center;
                                picture.HorizontalAlignment = ShapeHorizontalAlignment.Center;

                                if (picture.Height > picture.Width)
                                {
                                    var resizeFactor = picture.Height / 40;
                                    picture.Height = 40;
                                    picture.Width /= resizeFactor;
                                }
                                else
                                {
                                    var resizeFactor = picture.Width / 40;
                                    picture.Width = 40;
                                    picture.Height /= resizeFactor;
                                }

                                break;
                            default:
                                throw new NotImplementedException("Type not yet implemented {0}",
                                    cellContent.Content.GetType());
                        }

                        paragraph.Text = cellText;
                    }
                }

                var memoryStream = new MemoryStream();
                loadedDocument.Save(memoryStream, FormatType.Docx);
                memoryStream.Position = 0;

                var dialog = new SaveFileDialog
                {
                    InitialDirectory =
                        System.Environment.GetFolderPath(System.Environment.SpecialFolder.MyDocuments),
                    Filter = "Word bestand (*.docx)|*.docx"
                };


                if (dialog.ShowDialog() == true)
                {
                    using (memoryStream)
                    {
                        using (var fileStream = File.Create(dialog.FileName))
                        {
                            memoryStream.CopyTo(fileStream);
                        }
                    }
                }
            }
        }

        private Stream GetFromEmbeddedResources(string fileName)
        {
            var assembly = Assembly.GetExecutingAssembly();
            var resourceName = $"WordTestApp.{fileName}";

            var stream = assembly.GetManifestResourceStream(resourceName);
            if (stream == null)
                throw new NotImplementedException($"Embedded resource with name {fileName} was not found.");

            return stream;
        }
    }
}